// FileManager.cpp: implementation of the FileManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileManager.h"
#include "Utils.h"
#include "..\ImgSource\ISource.h" 

//#ifdef _DEBUG
//#undef THIS_FILE
//static char THIS_FILE[]=__FILE__;
//#define new DEBUG_NEW
//#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileManager::CFileManager()
{
	
}

CFileManager::~CFileManager()
{

}
bool CFileManager::OpenImageFile (LPCWSTR pszImageName ,ImageInfo* info)
{

	HISSRC hSrc;
	int iImageType ;
	HGLOBAL hOriginalRGB;
	UINT32 iOriginalWidth, iOriginalHeight;
	RGBQUAD Palette[256];

	USES_CONVERSION ; 
 
	TCHAR psPath [MAX_PATH] ; 
	CUtils::GetShortPathName(pszImageName,psPath) ; 

	hSrc = IS40_OpenFileSource(W2A(psPath)) ;  
	
	iImageType = IS40_GuessFileType(hSrc) ; 
		
	IS40_Seek(hSrc, 0, 0) ; 

	if (hSrc!= NULL) 
	{
	   switch (iImageType)
			{
				case IMAGE_TYPE_ERROR:
					break;
				case IMAGE_TYPE_BMP:
						hOriginalRGB = IS40_ReadBMP(hSrc, &iOriginalWidth, &iOriginalHeight, 24, Palette,0);
						break;
				case IMAGE_TYPE_JPG:
						hOriginalRGB = IS40_ReadJPG(hSrc, &iOriginalWidth, &iOriginalHeight, 24,0);
						break;
				case IMAGE_TYPE_PCX:
						hOriginalRGB = IS40_ReadPCX(hSrc, &iOriginalWidth, &iOriginalHeight, 24, Palette,0);
						break;
				case IMAGE_TYPE_PNG:
						hOriginalRGB = IS40_ReadPNG(hSrc, &iOriginalWidth, &iOriginalHeight, 24, Palette,0);
						break;
				case IMAGE_TYPE_TGA:
						hOriginalRGB = IS40_ReadTGA(hSrc, &iOriginalWidth, &iOriginalHeight, 24, Palette,1);
						break;
				case IMAGE_TYPE_TIFF:
						hOriginalRGB = IS40_ReadTIFF(hSrc, &iOriginalWidth, &iOriginalHeight, 24, Palette, 0,0);
						break;
				default:
				break;
			}
	 }
		
		IS40_CloseSource(hSrc) ;
	
		
		if (!hOriginalRGB)
		{
			
			/*CString strMsg ; 
			if (iImageType == IMAGE_TYPE_ERROR)
				strMsg.Format(CUtils::LoadResourceString(IDS_READ_ERROR), pszImageName);
			else
				strMsg.Format(CUtils::LoadResourceString(IDS_FORMAT_ERROR), pszImageName);*/
			GlobalFree(hOriginalRGB); //add ndz 29.11.2006 
			return false;
		}
		else {
			
			size_t buffsize = iOriginalHeight * 3 * iOriginalWidth ; 
			info->SetImage((LPBYTE)hOriginalRGB,buffsize) ; 		
			info->iSize = buffsize ; 
			info->nWidth = iOriginalWidth ; 
			info->nHeight = iOriginalHeight ; 
			info->nRowStride = iOriginalWidth * 3 ;
			info->nImageType = iImageType ; 
			info->nBytesPixel = 3 ; 
			wcscpy(info->szFile,pszImageName) ;
			GlobalFree(hOriginalRGB); 
			return true ;		
						
		}

} 

bool CFileManager::SaveImageFile (LPCWSTR pszFileName,ImageInfo* info) 
{
	RGBQUAD Palette[256];
	UINT32 iSize = 0 ; 

	USES_CONVERSION ; 
	
	TCHAR pszOut [MAX_PATH] ;  
	CUtils::GetShortPathName(pszFileName,pszOut) ; 
	HISDEST hDst = 	IS40_OpenFileDest(W2A(pszOut),FALSE);

	switch (info->nImageType)
	{
			case IMAGE_TYPE_ERROR:
				{
					
					CString strErrorMsg = _T("Unhandled exception") ; 
					UINT nLastError = IS40_GetLastError(); 

					CString strErrorNum ; strErrorNum.Format(_T("d%"),nLastError) ; 
					strErrorMsg += _T(" - error code - ") ; 
					strErrorMsg += strErrorNum ;
					AfxMessageBox(strErrorMsg)	;
					info->szErrorMsg  =  (LPCWSTR)strErrorMsg ; 
					return false ; 

				}
			break;
				case IMAGE_TYPE_BMP:
					{
					iSize = IS40_WriteBMP(hDst, (LPBYTE)info->hOriginalRGB ,info->nWidth,info->nHeight, 24,info->nWidth * 3, 0, Palette, NULL,0);
					}
					break;
				case IMAGE_TYPE_JPG:
					{
						iSize = IS40_WriteJPG(hDst, (LPBYTE)info->hOriginalRGB ,info->nWidth,info->nHeight,24,info->nWidth * 3 ,info->nJpegQuality, NULL,(1<<5));
						 IS40_ClearJPGOutputMarkers() ;
					}
					break;
				case IMAGE_TYPE_PCX:
					{
						iSize = IS40_WritePCX(hDst, (LPBYTE)info->hOriginalRGB ,info->nWidth,info->nHeight, 24,info->nWidth * 3, Palette,NULL,0);
					}
					break;
				case IMAGE_TYPE_PNG:
					{
						iSize =  IS40_WritePNG(hDst, (LPBYTE)info->hOriginalRGB ,info->nWidth,info->nHeight, info->nRowStride, 8, 0, Palette, 2, 0, 0, NULL);
					}
					break;
				case IMAGE_TYPE_TGA:
					{
						iSize = IS40_WriteTGA(hDst, (LPBYTE)info->hOriginalRGB ,info->nWidth,info->nHeight, 24,info->nWidth, Palette,NULL,0);
					}
					break;
				case IMAGE_TYPE_TIFF:
					{
					iSize = IS40_WriteTIFF(hDst,(LPBYTE)info->hOriginalRGB ,info->nWidth,info->nHeight, 24,info->nWidth * 3 , Palette, 1, NULL,0);
					}
					break;
				default:
				break;
			 }
	
		   // delete [] pByte  ;
			IS40_CloseDest(hDst) ;
			
		   	//GlobalFree(info->hOriginalRGB);
			//delete info->hOriginalRGB ; 
			
			
			if (iSize > 0 )
				return true ; 
			else 
			{
				
				CString strErrorMsg ; 
				UINT nLastError = IS40_GetLastError(); 
								
				CString lastErrorCode(_T(" : -err.code- ")) ; 
				lastErrorCode.Format(_T("%d"),nLastError) ; 
				

				if (nLastError == IS_ERR_FILEWRITE )
					strErrorMsg = _T("Cannot write to file.") ; 
					
				if (nLastError ==  IS_ERR_MEM) 
					strErrorMsg = _T("Out of memory.") ; 

				if (nLastError == IS_ERR_BADPARAM) 
					strErrorMsg = _T("Bad user parameter."); 

				strErrorMsg += lastErrorCode ; 
				info->szErrorMsg  =  (LPCWSTR)strErrorMsg ; 	
								
				return false ;

			}
		

}
UINT CFileManager::SetRequestedFileSize (UINT nMaxSize,ImageInfo& info) 
{
		
	TCHAR szPath[MAX_PATH];
	//SHGetSpecialFolderPath(NULL,szPath,CSIDL_MYPICTURES,NULL) ; 
	CString strPath (szPath) ; 
	strPath += _T("\temp.jpg") ; 
	UINT nRetSize = 0 ; 

	if (info.iSize < nMaxSize) 
	{
		return info.iSize ; 
	}

	//proveritit prvo da li ima smisla kompresovati sliku 
	LPCWSTR lpszPath = (LPCWSTR)strPath ; 
	USES_CONVERSION ; 
	
	TCHAR psPath [MAX_PATH] ; 
	CUtils::GetShortPathName(lpszPath,psPath) ;

	HISDEST hDest = IS40_OpenFileDest(W2A(psPath),FALSE);

	UINT iSize = IS40_WriteJPG(hDest, (LPBYTE)info.hOriginalRGB ,info.nWidth,info.nHeight,24,
										info.nWidth * 3 ,0, NULL,(1<<5));
	IS40_CloseDest(hDest) ; 

	if (iSize > nMaxSize)
	{
		return iSize ; 
	}

	
	UINT quality = info.nJpegQuality ; 
	//maximum 10 iteracija za jpeg kvalitet 		
	for(int i=0 ; i < 10 ;i++)
	{
			
		HISDEST hDest = IS40_OpenFileDest(W2A(psPath),FALSE);

		UINT iSize = IS40_WriteJPG(hDest, (LPBYTE)info.hOriginalRGB ,info.nWidth,info.nHeight,24,
										info.nWidth * 3 ,info.nJpegQuality, NULL,(1<<5));
		IS40_CloseDest(hDest) ; 
		
		if (iSize > nMaxSize)
		{
			info.nJpegQuality -= quality /=2 ;
			if (quality == 0 )
				info.nJpegQuality--; 
			
			CString strQuality ; 
			strQuality.Format(_T("%d"),info.nJpegQuality) ; 
			
			TRACE((LPCWSTR)strQuality) ; 
			TRACE(_T("\r\n")) ; 
				
		}
		else 
		{
			if ((double)iSize / (double)nMaxSize < 0.885f)
			{
				info.nJpegQuality += quality /=2 ; 
				if (quality == 0 )
					info.nJpegQuality++;
				
				CString strQuality ; 
				strQuality.Format(_T("%d"),info.nJpegQuality) ; 
				
				TRACE((LPCWSTR)strQuality) ; 
				TRACE(_T("\r\n")) ; 

			}
			else 
				break ; 
						
		}
		
	}

	return nRetSize;
}



CString CFileManager::FileOptions (BOOL m_bFileName,BOOL m_bSkipped,LPCWSTR pszFileName,
								   CString& m_strFolderPath,int nOperation )
{
	CString strFileName ;
	CString pszOutputFileName ; 

	return strFileName ; 
	
}
UINT CFileManager::GetFileType (LPCWSTR lpsFile) 
{
	
	USES_CONVERSION ; 
 
	TCHAR psPath [MAX_PATH] ; 
	CUtils::GetShortPathName(lpsFile,psPath) ; 

	HISSRC hSrc = IS40_OpenFileSource(W2A(psPath)) ;  
	
	UINT iImageType = IS40_GuessFileType(hSrc) ; 	

	IS40_CloseSource(hSrc) ; 
	
	return iImageType ; 

}
BOOL CFileManager::GetDims(CString strFile,UINT32* w,UINT32* h) 
{
	HISSRC hSrc;
	int iImageType ;
	
	UINT32  nBitDepth,nColorSpace;
	UINT32 nFlags ; 
	UINT32 nInterlace ,puInterlaceType ,nColorType;
    UINT32 puTransIndex ;
    UINT32 puTransRed;
    UINT32 puTransGreen; 
    UINT32 puTransBlue;
    UINT32 puTransGray;
	RGBQUAD Palette[256];

	BOOL bSucces = FALSE ; 

	USES_CONVERSION ; 
 
	TCHAR psPath [MAX_PATH] ; 
	CUtils::GetShortPathName(strFile,psPath) ; 

	hSrc = IS40_OpenFileSource(W2A(psPath)) ;  
	
	iImageType = IS40_GuessFileType(hSrc) ; 
		
	IS40_Seek(hSrc, 0, 0) ; 

	if (hSrc!= NULL) 
	{
	   switch (iImageType)
			{
				case IMAGE_TYPE_ERROR:
					break;
				case IMAGE_TYPE_BMP:
						bSucces = IS40_GetBMPDims(hSrc,w,h,&nBitDepth,NULL,0) ; 
						break;
				case IMAGE_TYPE_JPG:
						bSucces = IS40_GetJPGDims(hSrc,w,h,&nBitDepth,&nColorSpace,&nFlags,NULL,0) ; 
						break;
				case IMAGE_TYPE_PCX:
						bSucces = IS40_GetPCXDims(hSrc,w,h,&nBitDepth,NULL,0);
						break;
				case IMAGE_TYPE_PNG:
						 bSucces = IS40_GetPNGDims(hSrc,w,h,&nBitDepth,&nColorType,
										&puInterlaceType,&puTransIndex,
										&puTransRed,&puTransGreen,&puTransBlue,&puTransGray,NULL,0);
						break;
				case IMAGE_TYPE_TGA:
						bSucces = IS40_GetTGADims(hSrc,w,h,&nBitDepth,0);
						break;
				case IMAGE_TYPE_TIFF:
						bSucces = IS40_GetTIFFDims(hSrc,0,w,h,&nBitDepth,NULL,0);
						break;
				default:
				break;
			}
	 }
		
	IS40_CloseSource(hSrc) ;

	return bSucces ; 
    

}
LPBYTE CFileManager::GetThumbnail (LPCWSTR srcFile,UINT& w,UINT& h ) 
{
	   
	    USES_CONVERSION ; 

	   TCHAR psPath [MAX_PATH]; 
	   CUtils::GetShortPathName(srcFile,psPath) ; 

	
	  HISSRC hSrc = IS40_OpenFileSource(W2A(psPath));
	  HANDLE hEXIF = IS40_EXIFCreateReader(hSrc, 0, 1);
	  IS40_CloseSource(hSrc);
		
	  HGLOBAL hThumbnail = NULL;
	  HGLOBAL hImg=NULL ;
	  LPBYTE hThumb=NULL ; 

		UINT32 uTNLen = 0;
		if (hEXIF)
		{
			if (IS40_EXIFGetThumbnail(hEXIF, &hThumbnail, &uTNLen))
			{
				HISSRC hSrc =  IS40_OpenMemorySource((BYTE*)hThumbnail,uTNLen) ;  
				
				hImg = IS40_ReadImage(hSrc,&w,&h,0,0) ; 
							
				UINT nSize = w*h*3 ; 

				hThumb = DEBUG_NEW BYTE[nSize] ; 
				
				memcpy(hThumb,hImg,nSize) ; 

				IS40_CloseSource(hSrc) ; 

				GlobalFree(hImg) ; 
                        
			}
			
		}
		
		IS40_EXIFDestroyReader(hEXIF) ; 
		
    		
		return hThumb ; 
    

}
bool CFileManager::IsSuportedType (LPCWSTR pszFileName) 
{
	
	HISSRC hSrc;
	int iImageType ;

	USES_CONVERSION ; 
 	TCHAR psPath [MAX_PATH] ; 
	CUtils::GetShortPathName(pszFileName,psPath) ; 

	hSrc = IS40_OpenFileSource(W2A(psPath)) ;  
	iImageType = IS40_GuessFileType(hSrc) ;  
	IS40_CloseSource(hSrc) ; 
	
	if ( hSrc == NULL)
	{
		return false ; 
	}
	if (iImageType !=IMAGE_TYPE_ERROR) 
	{
		return true ; 
	}
	else 
	{
		return false ; 
	}
    

}
HBITMAP CFileManager::ExtractThumb(LPCWSTR szFile) 
{
		HRESULT hr ; 
		
		SIZE prgSize ; 
		prgSize.cx = 160 ; 
		prgSize.cy = 120; 
		DWORD dwRecClrDepth = 3 ; 

		ImageInfo info ; 
		CFileManager manager ; 
			
		CoInitialize(NULL) ; 
		IShellFolder* pShell = 0 ;
		ULONG nItems = 1 ; 
		
		hr = SHGetDesktopFolder(&pShell) ;  

		CString  strpath = CUtils::GetFullPath(szFile) ; 
		CString strFile =  CUtils::GetFileExt(szFile) ; 
		
		ATLTRACE(strpath) ;
		ATLTRACE(_T("\r\n")) ; 

		LPWSTR path = strpath.GetBuffer(1);  
		ATLTRACE(path) ; 
		ATLTRACE(_T("\r\n")) ; 
					
		ATLTRACE(W2OLE(path)) ; 
		ATLTRACE(_T("\r\n")) ;

		LPITEMIDLIST pidl = NULL; // general purpose
		
		hr = pShell->ParseDisplayName(NULL, NULL,W2OLE(path), NULL, &pidl, NULL);

		if (hr != S_OK) 
		{
			TRACE(_T("ParseDisplayName failed \r\n")) ; 
			return NULL; 
		}

		LPSHELLFOLDER psfFolder = NULL;
		hr = pShell->BindToObject(pidl,NULL,IID_IShellFolder,(void**)&psfFolder) ;

		if (hr != S_OK) 
		{
			TRACE(_T("BindToObject failed \r\n")) ; 
			return NULL; 
		}
		
		LPWSTR file  = strFile.GetBuffer(1);
	
		psfFolder->ParseDisplayName(NULL,NULL,W2OLE(file),NULL,&pidl,NULL) ; 
		if (hr != S_OK) 
		{
			TRACE(_T("ParseDisplayName failed \r\n")) ; 
			return NULL; 
		}

		IExtractImage* pIExtract = NULL;
		LPCITEMIDLIST pidlExtract = pidl ; 
				  
		hr = psfFolder->GetUIObjectOf(NULL, 1, &pidlExtract, IID_IExtractImage,
												 NULL, (void**)&pIExtract);
		if(NULL == pIExtract) // early shell version, thumbs not supported
			  return NULL;

		OLECHAR wszPathBuffer[MAX_PATH];
		DWORD dwPriority = 0; // IEI_PRIORITY_NORMAL is defined nowhere!
		DWORD dwFlags = IEIFLAG_SCREEN;
				   
		HBITMAP hBmpImage = NULL;
				   
		hr = pIExtract->GetLocation(wszPathBuffer, MAX_PATH, &dwPriority,
												&prgSize, dwRecClrDepth, &dwFlags);
				   
				   // even if we've got shell v4.70+, not all files support thumbnails 
		if(NOERROR == hr) 
		  
			hr = pIExtract->Extract(&hBmpImage);
				   
		 pIExtract->Release();
		 return hBmpImage;		



}


int CFileManager::GetEXIFString(LPCTSTR pFileName, UINT32 nTag)
{
	USES_CONVERSION ; 
 	TCHAR psPath [MAX_PATH] ; 
	CUtils::GetShortPathName(pFileName,psPath) ; 

 	int nRetVal = 0;
	HISSRC hSrc = IS40_OpenFileSource(W2A(psPath));
	
	HISEXIFREADER hEXIF = IS40_EXIFCreateReader(hSrc, 0, 1);
	IS40_CloseSource(hSrc);
	

	HGLOBAL hStr = NULL;
    UINT32 uStrLen;
	IS40_EXIFGetString(hEXIF, nTag, 31, &hStr, &uStrLen); 
    
	IS40_EXIFDestroyReader(hEXIF) ;
	if (hStr != NULL) 
	{
		char* pStr = (char*)hStr ; 
		nRetVal = atoi((char*)hStr) ;
		GlobalFree(hStr);
		return  nRetVal ; 
	}
	else 
	{
		GlobalFree(hStr);
		return 0 ; 
	}

	return 0;
}
